﻿using System;
using System.Diagnostics;
using System.Collections.Generic;
using System.Collections.ObjectModel;
using System.Linq;
using System.Text;
using StreamBase.SB;
using StreamBase.DataCache;

namespace DataCacheBrowser
{
    class GenericModel : ObservableCollection< object >, IDataCacheModel
    {
        public GenericModel()
        {
        }

        /*
         * Normally this would inherit from ObservableObject (see GenericView) and would
         * have a series of properties that notify when they change -- this
         * allows them to be observable directly
         */

        public void UpdateFromTuple(StreamBase.SB.Tuple t)
        {
            object[] fields = ConvertTuple(t);

            // Initialize if null from scratch
            if ( Count == 0 ) {
                for ( int i = 0; i < t.GetSchema().GetFieldCount(); i ++ )
                    Add( fields[i] );
                return;
            }

            // Update otherwise
            for ( int i = 0; i < t.GetSchema().GetFieldCount(); i ++ ) {
                if ( this[i] != null && this[i].Equals( fields[i] ) )
                    continue;
                this[i] = fields[i];
            }
        }

        private object[] ConvertTuple(StreamBase.SB.Tuple _data)
        {
 
            object[] output = new object[ _data.GetSchema().GetFieldCount() ];

            for ( int idx = 0; idx < _data.GetSchema().GetFieldCount(); idx ++ ) {

                Schema.Field f = _data.GetSchema().GetField(idx);
                if (f.GetDataType() == DataType.TUPLE)
                {
                    output[idx] = _data.GetField(idx).ToString();
                }
                else if (f.GetDataType() == DataType.LIST)
                {
                    StringBuilder sb = new StringBuilder("[");
                    foreach (object s in _data.GetList(idx))
                    {
                        if (sb.Length > 1)
                            sb.Append("],[");
                        sb.Append(s);
                    }
                    sb.Append("]");
                    output[idx] = sb.ToString();
                }
                else
                {
                    output[idx] = _data.GetField(idx);
                }
            }

            return output;
        }
    }
}
