﻿using System;
using System.Timers;
using System.Collections.Generic;
using System.Collections.ObjectModel;
using System.Linq;
using System.Text;
using System.Windows;
using System.Windows.Controls;
using System.Windows.Data;
using System.Windows.Documents;
using System.Windows.Input;
using System.Windows.Media;
using System.Windows.Media.Imaging;
using System.Windows.Navigation;
using System.Windows.Shapes;
using StreamBase.SB;
using StreamBase.SB.Client;
using Microsoft.Windows.Controls;
using AvalonDock;
using System.Diagnostics;
using System.ComponentModel;
using System.Windows.Threading;
using System.CodeDom;
using StreamBase.DataCache;

/*
 * Steps:
 * 
 * Generic for output:
 * 1. Get the Table class working generically for all data views (code it directly into it -- XAML is less important for this one)
 * 2. Get the automatic detection of connections and available containers
 * 3. Get the automatic detection of available output caches
 * 4. Create the menu based on available output caches and adjust
 * 5. Create the updating of caches based on underlying availability -- always try to subscribe -- and work it like that.
 * 6. Get unsubscription and re-subscription and all of that jazz working
 * 
 * Generic for input:
 * 1. Get something similar for input views - automatically creating the input parameters
 * 
 * Ultimately though a few other questions:
 * - Authorizing limited views (read-only, read-write to certain channels, etc) -> this may be controlled server side for real! Ideal way.
 * - Customized views for specific data, browsing, etc, etc
 * - Server side persistence/restoring for local layout and other things
 *   i.e., centralized configuration management, with local UIs able to read-write certain aspects and just read other
 */

namespace DataCacheBrowser
{
    /// <summary>
    /// Interaction logic for Window1.xaml
    /// </summary>
    /// 

    public partial class GenericView : DockableContent /*, IDataCacheConsumer */
    {
        public GenericView(string cacheName)
        {
            InitializeComponent();

            this.Title = cacheName;

            /* New DataCache Model Collection based on the GenericModel! */
            GenericViewModel gv = new GenericViewModel();
            gv.DataModel = new DataCacheModelCollection< GenericModel >( cacheName );
            gv.DataModel.SchemaHandler += initGrid;
            this.myGrid.DataContext = gv;

            /* Periodically refresh to ensure sort order */
            Timer _timer = new Timer();
            _timer.Interval = 500;
            _timer.Elapsed += new ElapsedEventHandler(TimerEvent);
            _timer.Start();
        }

        private void TimerEvent(object sender, EventArgs e)
        {
            if (Application.Current == null)
            {
                return;
            }
            if (!Application.Current.Dispatcher.CheckAccess())
            {
                Application.Current.Dispatcher.BeginInvoke(DispatcherPriority.Normal, new System.Windows.Forms.MethodInvoker(Refresh));
                return;
            }

            Refresh();
        }

        private void Refresh()
        {
            try
            {
                if (this.myGrid.IsVisible)
                {
                    this.myGrid.Items.Refresh();
                }
            }
            catch (Exception ex)
            {
                Debug.WriteLine(ex.ToString());
            }
        }

        private void initGrid(Schema schema)
        {
            myGrid.Columns.Clear();
            for (int i = 0; i < schema.GetFieldCount(); i++)
            {
                Schema.Field f = schema.GetField(i);
                
                Binding b = new Binding();
                b.Mode = BindingMode.OneWay;
                b.Path = new PropertyPath("[" + i + "]");//Columns[" + i + "]");
                
                DataGridTextColumn c = new DataGridTextColumn();
                c.Header = f.GetName();
                c.Binding = b;
                c.CanUserSort = true;
                myGrid.Columns.Add(c);
            }
        }
    }
}
