package com.streambase.contrib.datacache.gui.swing;


import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.awt.event.KeyEvent;
import java.util.Collections;

import javax.swing.JFrame;
import javax.swing.JMenu;
import javax.swing.JMenuBar;
import javax.swing.JMenuItem;
import javax.swing.SwingUtilities;

import bibliothek.gui.dock.common.CControl;
import bibliothek.gui.dock.common.DefaultSingleCDockable;

import com.streambase.contrib.datacache.DataCacheService;
import com.streambase.contrib.datacache.gui.SharedUtilities;

/**
 * Sample Swing UI capable of displaying a table for any DataCache currently running
 * in a StreamBase application.
 * 
 * @author mark.scannell@streambase.com
 */
public class DataCacheSwingMain {


	/**
	 * @param args
	 */
	public static void main(String[] args) {
		// Set preferred look and feel
		try {
			javax.swing.UIManager.setLookAndFeel("com.sun.java.swing.plaf.nimbus.NimbusLookAndFeel");
		} catch ( Exception e ) {
			// ignore
		}
		
		// Connect
		SharedUtilities.initializeServerConnectivity(args);

		// Start the GUI in the UI thread
		SwingUtilities.invokeLater( new Runnable() {
			@Override
			public void run() {
				runGUI();
			}
		});
	}

	private static void runGUI() {
		
		JMenuBar menuBar;
		JMenuItem menuItem;

		//Create the menu bar.
		menuBar = new JMenuBar();

		//Build the first menu.
		JMenu menu = new JMenu("View");
		menu.setMnemonic(KeyEvent.VK_V);
		menu.getAccessibleContext().setAccessibleDescription("The only menu in this program that has menu items");
		menuBar.add(menu);
		
		JFrame jf = new JFrame();
		final CControl c = new CControl( jf );

		// Initialize the View menu from the available caches. Kind of ugly.
		java.util.List< String > sortedCaches = new java.util.ArrayList< String >( DataCacheService.getDataCaches() );
		Collections.sort( sortedCaches );
		for ( String m : sortedCaches ) {
			final String cc[] = m.split("\\.",2);
			final String mm = m;
			menuItem = new JMenuItem( mm );
			menuItem.addActionListener( new ActionListener() {
				@Override
				public void actionPerformed(ActionEvent arg0) {
					DefaultSingleCDockable scd = (DefaultSingleCDockable)c.getSingleDockable( mm );
					if ( scd == null ) {
						scd = new DefaultSingleCDockable( mm, cc[1], new DataCacheTable( mm ) );
						c.add(scd);
					}
					scd.setVisible(true);
				}
				
			});
			menu.add( menuItem );
		}
		
		// Start it up
		jf.setJMenuBar( menuBar );
		jf.setDefaultCloseOperation( JFrame.EXIT_ON_CLOSE );
		jf.add( c.getContentArea() );
		jf.setSize( 400, 400 );
		jf.setVisible( true );
	}
}
